class BulkCreateMixin:
    """Allows bulk creation of a resource."""
    def get_serializer(self, *args, **kwargs):
        if isinstance(kwargs.get('data', {}), list):
            kwargs['many'] = True
        return super().get_serializer(*args, **kwargs)



from django.contrib import admin

from . import models


@admin.register(models.User)
class UserAdmin(admin.ModelAdmin):
    search_fields = ['username', 'email', 'first_name', 'last_name']
    list_display = ['username', 'email', 'first_name', 'last_name', 'is_staff']
    list_filter = ['is_staff', 'is_active']
    ordering = ['username']




from django.conf import settings
from django.contrib.auth.tokens import default_token_generator
from djoser import utils
from djoser.email import BaseEmailMessage


class PasswordResetEmail(BaseEmailMessage):
    template_name = 'email/password_reset.html'

    def get_context_data(self):
        context = super(PasswordResetEmail, self).get_context_data()

        user = context.get('user')
        context['uid'] = utils.encode_uid(user.pk)
        context['token'] = default_token_generator.make_token(user)
        context['site_name'] = settings.SITE_NAME
        context['domain'] = settings.SITE_DOMAIN
        context['protocol'] = settings.SITE_PROTOCOL
        context['url'] = settings.PASSWORD_RESET_CONFIRM_URL.format(**context)
        return context



from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from django.contrib.auth.models import AnonymousUser


class NoFailJWTAuthentication(JSONWebTokenAuthentication):
    """
    A jwt authentication method that doesn't fail
    """

    def authenticate(self, *args, **kwargs):
        try:
            return super(NoFailJWTAuthentication, self).authenticate(*args, **kwargs)
        except:
            return (AnonymousUser(), None)



from django.contrib import admin

from .models import *


admin.site.register(Annotation)
admin.site.register(Entity)
admin.site.register(Annotator)
admin.site.register(Corpus)
admin.site.register(Document)
admin.site.register(DocumentScore)
admin.site.register(DocumentStatus)
admin.site.register(Task)
admin.site.register(Organization)
admin.site.register(Collaborator)



from datetime import datetime
from django.contrib.auth.models import AbstractUser, AnonymousUser
from django.db.models.signals import post_save
from django.core.mail import EmailMessage
from django.template import loader
from django.db import models
from django.conf import settings
from django.dispatch import receiver
from django.core.mail import send_mail
from django.template.loader import render_to_string


def avatar_directory_path(instance, filename):
    return 'avatar/{0}_{1}'.format(datetime.timestamp(datetime.now()), filename)

class User(AbstractUser):
    email = models.EmailField(unique=True)
    avatar = models.ImageField(
        upload_to=avatar_directory_path,
        null=True,
        blank=True,
        height_field=None,
        width_field=None,
        max_length=None)


class CustomAnonymousUser(AnonymousUser):
    email = 'anonymous@linalgo.com'


@receiver(post_save, sender='linauth.User')
def on_new_user(sender, instance, created, signal, *args, **kwargs):
    if created:
        subject_template = 'email/new_user_notification_subject.txt'
        body_template = 'email/new_user_notification_body.html'

        from_email = settings.NEW_USER_NOTIFICATION_FROM_EMAIL
        to_emails = settings.NEW_USER_NOTIFICATION_TO_EMAILS
        context = {'user': instance}

        subject = render_to_string(subject_template, context).strip()
        message = render_to_string(body_template, context)

        send_mail(
            subject=subject,
            message=message,
            from_email=from_email,
            recipient_list=to_emails,
            html_message=message
        )



from linhub import backends, models


def invite_collaborator(email, role, organization, request=None):
    if email is not None:
        backend = backends.LinHubInvitationBackend()
        user = backend.invite_by_email(
            email=email,
            domain=request.get_host,
            organization=organization,
            sender=request.user)
    collaborator, _ = models.Collaborator.objects.get_or_create(
        user=user, organization=organization)
    collaborator.role = role
    collaborator.save()
    return collaborator



import hashlib

from django.contrib.auth import get_user_model

from rest_framework.authtoken.models import Token
from rest_framework import serializers


class UserSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = get_user_model()
        fields = ('id', 'username', 'first_name', 'last_name', 'email',
                  'avatar', 'date_joined', 'is_active')
        read_only_fields = ['is_active', 'date_joined']


class TokenSerializer(serializers.ModelSerializer):
    class Meta:
        model = Token
        fields = ('key', 'created',)



import requests

from django.contrib.auth import get_user_model
from django.core.mail import EmailMessage
from django.template import loader
from django.urls import reverse

from rest_framework import generics, status
from rest_framework.response import Response
from rest_framework.permissions import AllowAny
from djoser.serializers import UserCreateSerializer
from djoser.compat import get_user_email
from djoser import signals
from djoser.conf import settings as dj_settings

from . import models, serializers


def create_user_and_organization(username, first_name, last_name, email, password=None):
    user = get_user_model().objects.get(email=email)
    user.username = username
    user.first_name = first_name
    user.last_name = last_name
    if password is not None:
        user.set_password(password)
    user.save()
    organization, _ = models.Organization.objects.get_or_create(
        name=username, email=email, individual=True, person=user)
    organization.save()
    collaborator = models.Collaborator(
        user=user, organization=organization, role=models.Collaborator.OWNER)
    collaborator.save()
    annotator, created = models.Annotator.objects.get_or_create(
        owner=user, model=models.Annotator.HUMAN)
    annotator.name = username
    annotator.save()
    return user


class RegistrationView(generics.GenericAPIView):
    serializer_class = serializers.UserRegistrationSerializer
    permission_classes = (AllowAny, )

    def post(self, request):
        serializer = serializers.UserRegistrationSerializer(data=request.data)
        if serializer.is_valid():

            user_s = UserCreateSerializer(data=serializer.data)

            try:
                user_s.is_valid(raise_exception=True)

                user = user_s.save()
                signals.user_registered.send(
                    sender=self.__class__, user=user, request=self.request
                )

                context = {'user': user}
                to = [get_user_email(user)]
                if dj_settings.SEND_ACTIVATION_EMAIL:
                    dj_settings.EMAIL.activation(self.request, context)\
                        .send(to)
                elif dj_settings.SEND_CONFIRMATION_EMAIL:
                    dj_settings.EMAIL.confirmation(self.request, context)\
                        .send(to)
                
                user = create_user_and_organization(
                    username=serializer.validated_data['username'],
                    first_name=serializer.validated_data['first_name'],
                    last_name=serializer.validated_data['last_name'],
                    email=serializer.validated_data['email'])
                self.email_message(user)
                return Response(serializer.data, 
                    status=status.HTTP_201_CREATED)
            except Exception as e:
                return Response(user_s.errors, 
                    status=status.HTTP_400_BAD_REQUEST)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def email_message(self, user):

        subject_template = 'email/welcome_subject.txt'
        body_template = 'email/welcome_body.html'

        from_email = 'Linalgo <registration@linalgo.com>'
        reply_to = ['Linalgo <support@linalgo.com>']
        context = {'user': user, 'domain': 'https://linhub.net'}

        subject_template = loader.get_template(subject_template)
        body_template = loader.get_template(body_template)
        subject = subject_template.render(context).strip()
        body = body_template.render(context)
        msg = EmailMessage(
            subject, body, from_email, [user.email], reply_to=reply_to)
        msg.send(fail_silently=False)


class ActivationView(generics.GenericAPIView):
    serializer_class = serializers.UserActivationSerializer
    permission_classes = (AllowAny, )

    def post(self, request):
        serializer = serializers.UserActivationSerializer(data=request.data)
        if serializer.is_valid():
            url = 'http://' + request.get_host() + '/auth/users/activate'
            response = requests.post(url, data=serializer.data)
            if response.status_code == status.HTTP_204_NO_CONTENT:
                user = create_user_and_organization(
                    username=serializer.validated_data['username'],
                    first_name=serializer.validated_data['first_name'],
                    last_name=serializer.validated_data['last_name'],
                    email=serializer.validated_data['email'],
                    password=serializer.validated_data['password'])
                self.email_message(user)
                return Response(serializer.data, status=response.status_code)
            else:
                return Response(response.json(), response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def email_message(self, user):

        subject_template = 'email/welcome_subject.txt'
        body_template = 'email/welcome_body.html'

        from_email = 'Linalgo <registration@linalgo.com>'
        reply_to = ['Linalgo <support@linalgo.com>']
        context = {'user': user, 'domain': 'https://linhub.net'}

        subject_template = loader.get_template(subject_template)
        body_template = loader.get_template(body_template)
        subject = subject_template.render(context).strip()
        body = body_template.render(context)
        msg = EmailMessage(
            subject, body, from_email, [user.email], reply_to=reply_to)
        msg.send(fail_silently=False)


class AskInvitationView(generics.GenericAPIView):
    serializer_class = serializers.AskInvitationSerializer
    permission_classes = (AllowAny, )

    def post(self, request):
        serializer = serializers.AskInvitationSerializer(data=request.data)
        if serializer.is_valid():
            user = {
                'username': serializer.validated_data['name'],
                'email': serializer.validated_data['email'],
                'company': serializer.validated_data['company'],
                'role': serializer.validated_data['role']
            }
            self.email_message(user)
            return Response(status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def email_message(self, user):

        subject_template = 'email/ask4invite_subject.txt'
        body_template = 'email/ask4invite_body.html'

        from_email = 'Linalgo <invite@linalgo.com>'
        reply_to = ['Linalgo <invite@linalgo.com>']
        to_emails = [user['email'], from_email]
        context = {'user': user, 'domain': 'https://linhub.net'}

        subject_template = loader.get_template(subject_template)
        body_template = loader.get_template(body_template)
        subject = subject_template.render(context).strip()
        body = body_template.render(context)
        msg = EmailMessage(
            subject, body, from_email, to_emails, reply_to=reply_to)
        msg.content_subtype = "html"
        msg.send(fail_silently=False)
